from django.contrib import admin
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.shortcuts import redirect, reverse
from django.template.response import TemplateResponse
from django.urls import path
from django.contrib import messages
from drf_user_activity_tracker_mongodb.utils import (MyCollection,
                                                     get_all_url_names,
                                                     ParamsHandler)
from drf_user_activity_tracker_mongodb.utils import database_log_enabled

if database_log_enabled():
    from drf_user_activity_tracker_mongodb.models import ActivityLog


    @admin.register(ActivityLog)
    class ActivityLogAdmin(admin.ModelAdmin):
        model = ActivityLog

        def get_urls(self):
            info = self.model._meta.app_label, self.model._meta.model_name
            print('%s_%s_changelist' % info)

            return [
                path(r'', self.admin_site.admin_view(self.activity_log_view), name='%s_%s_changelist' % info),
                path(r'<str:pk>/change/',
                     self.admin_site.admin_view(self.activity_log_detail_view),
                     name='%s_%s_change' % info)
            ]

        def activity_log_view(self, request):
            params = ParamsHandler(request)

            url_names_list = get_all_url_names()
            url_name = params.get_url_name()
            search_value = params.get_search_value()
            status_code = params.get_status()
            time_delta = params.get_time_delta()

            page = request.GET.get('page', 1)

            dataset = MyCollection().list(url_name=url_name, user_id=search_value, status_code=status_code,
                                          time_delta=time_delta)
            paginator = Paginator(dataset, 50)

            try:
                page_object = paginator.page(page)
            except PageNotAnInteger:
                page_object = paginator.page(1)
            except EmptyPage:
                page_object = paginator.page(paginator.num_pages)

            context = dict(self.admin_site.each_context(request), dataset=page_object, url_names=url_names_list,
                           count=paginator.count)
            return TemplateResponse(request, "activity_log/admin/change_list.html", context=context)

        def activity_log_detail_view(self, request, pk=None):
            data = MyCollection().detail(pk)
            if not data:
                messages.warning(request, f"Log with ID '{pk}' doesn’t exist. Perhaps it was deleted?")
                return redirect(reverse('admin:index'))
            context = dict(self.admin_site.each_context(request), data=data)
            return TemplateResponse(request, "activity_log/admin/change_detail.html", context=context)



from urllib.parse import urlencode

from django import template

register = template.Library()


@register.simple_tag
def under_score_tag(obj, attribute):
    obj = dict(obj)
    return obj.get(attribute)


@register.simple_tag
def clean_url_encode(data, remove):
    attrs = data.copy()
    if remove in data:
        attrs.pop(remove)
    return urlencode(attrs)



from django.conf import settings
from rest_framework import serializers


class ActivityLogSerializer(serializers.Serializer):
    _id = serializers.CharField()
    event_name = serializers.SerializerMethodField(read_only=True)
    created_time = serializers.DateTimeField()
    client_ip_address = serializers.CharField()
    url_name = serializers.CharField(max_length=255, write_only=True)

    class Meta:
        fields = ['_id', 'event_name', 'client_ip_address', 'created_time']

    def get_event_name(self, obj):
        event_name = obj.get('url_name', '')
        if hasattr(settings, 'DRF_ACTIVITY_TRACKER_EVENT_NAME'):
            if isinstance(settings.DRF_ACTIVITY_TRACKER_EVENT_NAME, dict):
                event_name = settings.DRF_ACTIVITY_TRACKER_EVENT_NAME.get(obj.get('url_name'), obj.get('url_name'))
                if not event_name:
                    event_name = obj.get('url_name', '')
        return event_name


class ActivityLogAdminSerializer(ActivityLogSerializer):
    user_id = serializers.IntegerField()

    class Meta:
        fields = ['_id', 'user_id', 'event_name', 'client_ip_address', 'created_time']



from rest_framework.generics import GenericAPIView
from rest_framework.permissions import IsAuthenticated
from rest_framework_simplejwt.authentication import JWTAuthentication
from rest_framework.response import Response

from drf_user_activity_tracker_mongodb.utils import MyCollection
from drf_user_activity_tracker_mongodb.serializers import ActivityLogSerializer, ActivityLogAdminSerializer


class ActivityLogView(GenericAPIView):
    authentication_classes = [JWTAuthentication]
    permission_classes = [IsAuthenticated]
    serializer_class = ActivityLogSerializer

    def get_serializer_class(self):
        serializer = self.serializer_class
        if self.request.user.is_superuser:
            serializer = ActivityLogAdminSerializer
        return serializer

    def get(self, request):
        if not self.request.user.is_superuser:
            response = MyCollection().list(user_id=self.request.user.id, api=True)
        else:
            response = MyCollection().list(api=True)

        serializer = self.get_serializer(instance=response, many=True)
        return Response(serializer.data)





import importlib
import re

from django.contrib import messages
from bson import ObjectId
from django.apps import apps
from django.conf import settings
from django.utils import timezone
from pymongo import MongoClient

SENSITIVE_KEYS = ['password', 'token', 'access', 'refresh']
if hasattr(settings, 'DRF_ACTIVITY_TRACKER_EXCLUDE_KEYS'):
    if isinstance(settings.DRF_ACTIVITY_TRACKER_EXCLUDE_KEYS, (list, tuple)):
        SENSITIVE_KEYS.extend(settings.DRF_ACTIVITY_TRACKER_EXCLUDE_KEYS)


def get_headers(request=None):
    """
        Function:       get_headers(self, request)
        Description:    To get all the headers from request
    """
    regex = re.compile('^HTTP_')
    return dict((regex.sub('', header), value) for (header, value)
                in request.META.items() if header.startswith('HTTP_'))


def get_client_ip(request):
    try:
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            ip = x_forwarded_for.split(',')[0]
        else:
            ip = request.META.get('REMOTE_ADDR')
        return ip
    except:
        return ''


def is_activity_tracker_enabled():
    drf_activity_tracker_database = False
    if hasattr(settings, 'DRF_ACTIVITY_TRACKER_DATABASE'):
        drf_activity_tracker_database = settings.DRF_ACTIVITY_TRACKER_DATABASE

    drf_activity_tracker_signal = False
    if hasattr(settings, 'DRF_ACTIVITY_TRACKER_SIGNAL'):
        drf_activity_tracker_signal = settings.DRF_ACTIVITY_TRACKER_SIGNAL
    return drf_activity_tracker_database or drf_activity_tracker_signal


def database_log_enabled():
    drf_activity_tracker_database = False
    if hasattr(settings, 'DRF_ACTIVITY_TRACKER_DATABASE'):
        drf_activity_tracker_database = settings.DRF_ACTIVITY_TRACKER_DATABASE
    return drf_activity_tracker_database


def mask_sensitive_data(data):
    """
    Hides sensitive keys specified in sensitive_keys settings.
    Loops recursively over nested dictionaries.
    """

    if not isinstance(data, dict):
        return data

    for key, value in data.items():
        if key in SENSITIVE_KEYS:
            data[key] = "***FILTERED***"

        if isinstance(value, dict):
            data[key] = mask_sensitive_data(data[key])

        if isinstance(value, list):
            data[key] = [mask_sensitive_data(item) for item in data[key]]

    return data


class MongoConnection(object):
    def __init__(self):
        client = MongoClient(settings.DRF_ACTIVITY_TRACKER_MONGO_CONNECTION)

        self.db = client[settings.DRF_ACTIVITY_TRACKER_MONGO_DB_NAME]
        self.collection = self.db[settings.DRF_ACTIVITY_TRACKER_MONGO_DB_COLLECTION_NAME]


class MyCollection(MongoConnection):

    def save(self, obj_list):
        self.collection.insert_many(obj_list)

    def list(self, user_id=None, url_name=None, status_code=None, time_delta=None, api=False):
        filter_params = {}
        if user_id:
            filter_params.update({'user_id': user_id})
        if api:
            limit = 100
            if hasattr(settings, 'DRF_ACTIVITI_API_LIMIT'):
                if isinstance(settings.DRF_ACTIVITI_API_LIMIT, (tuple, list)):
                    limit = settings.DRF_ACTIVITI_API_LIMIT
            return list(self.collection.find(filter_params).sort('_id', -1).limit(limit))
        if url_name:
            filter_params.update({'url_name': url_name})
        if status_code:
            filter_params.update({'status_code': {'$gte': status_code, '$lt': status_code + 100}})
        if time_delta:
            filter_params.update({'created_time': time_delta})
        return list(self.collection.find(filter_params).sort('_id', -1))

    def detail(self, pk):
        try:
            pk = ObjectId(pk)
        except:
            return None
        return self.collection.find_one({'_id': pk})


def get_all_url_names():
    list_of_all_urls = list()
    list_of_url_name = list()
    for name, app in apps.app_configs.items():
        mod_to_import = f'{name}.urls'
        try:
            urls = getattr(importlib.import_module(mod_to_import), 'urlpatterns')
            list_of_all_urls.extend(urls)
        except ImportError as ex:
            # is an app without urls
            pass
    for url in list_of_all_urls:
        if hasattr(url, 'name'):
            list_of_url_name.append(url.name)
    return list_of_url_name


class ParamsHandler:

    def __init__(self, request):
        self.params = request.GET

    def get_url_name(self):
        return self.params.get('url_name')

    def get_search_value(self):
        search_params = self.params.get('q')
        if search_params and search_params.isdigit():
            return int(search_params)

    def get_status(self):
        status_code = self.params.get('status')
        if status_code and status_code.isdigit():
            return int(status_code)

    def get_time_delta(self):
        now = timezone.now()
        today = now.today()
        return {
            'today': {'$lte': now, '$gt': today.replace(hour=0, minute=0, second=0, microsecond=0)},
            'past_7_days': {'$lte': now,
                            '$gt': today.replace(hour=0, minute=0, second=0, microsecond=0) - timezone.timedelta(
                                days=7)},
            'this_month': {'$lte': today,
                           '$gt': today.replace(day=1, hour=0, minute=0, second=0, microsecond=0)},
            'this_year': {'$lte': today,
                          '$gt': today.replace(month=1, day=1, hour=0, minute=0, second=0, microsecond=0)},
        }.get(self.params.get('created_time'))



from django.utils.translation import ugettext_lazy as _
from django.db import models

from drf_user_activity_tracker_mongodb.utils import database_log_enabled

if database_log_enabled():
    """
    Load models only if DRF_ACTIVITY_TRACKER_DATABASE is True
    """

    class ActivityLog(object):
        class Meta(object):
            app_label = 'drf_user_activity_tracker_mongodb'
            object_name = 'activity_log'
            model_name = module_name = 'activity_log'
            verbose_name = _('activity log')
            verbose_name_plural = _('activity logs')
            abstract = False
            swapped = False
            app_config = ""

        _meta = Meta()
        objects = models.Manager()



from django.urls import path
from drf_user_activity_tracker_mongodb.views import ActivityLogView

app_name = "activity_log"


urlpatterns = [
    path('history/', ActivityLogView.as_view(), name='history')
]


